#!/usr/local/bin/python
# -*- coding: utf-8 -*-

import treelib
import ety


class EtyTree(treelib.Tree):
    def __init__(self, word):
        if not isinstance(word, ety.Word):
            raise TypeError("word must be an instance of 'ety.Word'")
        self.source_word = word

        super(EtyTree, self).__init__()

        if not self.source_word.origins():
            return

        self.create_node(word.pretty, word._id, data=word.__dict__)

        self.add_children(self.source_word)

    def add_children(self, parent):
        for origin in parent.origins():
            key = origin._id

            try:
                self.create_node(
                    origin.pretty, key, parent=parent._id, data=origin.__dict__
                )
            except treelib.exceptions.DuplicatedNodeIdError:
                continue

            self.add_children(origin)

    def __bool__(self):
        return bool(self.source_word.origins())

    def __str__(self):
        try:
            return super(EtyTree, self).__str__().strip()
        except treelib.exceptions.NodeIDAbsentError:
            return ""

    def __repr__(self):
        return u"EtyTree(word='{}')".format(self.source_word.word)

    @property
    def __dict__(self):
        return self.to_dict(with_data=True)



#!/usr/local/bin/python
# -*- coding: utf-8 -*-

from .data import iso_639_3_codes, iso_639_2_codes


class Language(object):
    def __init__(self, iso):
        self.iso = iso

        try:
            self.name = iso_639_3_codes[iso]
        except KeyError:
            try:
                self.name = iso_639_2_codes[iso]
            except KeyError:
                raise KeyError("Language with iso code '%s' unknown" % iso)

    def __repr__(self):
        return u"Language(iso={})".format(self.iso)

    def __str__(self):
        return self.name

    def __eq__(self, other):
        return self.iso == other.iso and self.name == other.name



"""
This script downloads source data from data.jmsv.me; this contains an unzipped
and filtered mirror of the http://www1.icsi.berkeley.edu/~demelo/etymwn dataset

Data was filtered using the script at: https://data.jmsv.me/etymwn-filterer.sh
"""

import csv
import hashlib
import os
import io
import json
import gc

import requests
import six
from clint.textui import progress


def prepare(source_dir):
    """
    Create data source directory if not exists
    """
    if not os.path.exists(source_dir):
        os.makedirs(source_dir)


def download_dataset(url, dl_path):
    """
    Download filtered etymwn from jmsv.me mirror, displaying progress bar
    """
    r = requests.get(url, stream=True)

    with open(dl_path, "wb") as f:
        total_length = int(r.headers.get("content-length"))
        chunk_size = 4096
        for chunk in progress.bar(
            r.iter_content(chunk_size=chunk_size),
            expected_size=(total_length / chunk_size) + 1,
        ):
            if chunk:
                f.write(chunk)
                f.flush()
    print("Downloaded to " + dl_path)


def verify_local_data(url, dl_path):
    """
    Compare actual file checksum with expected served checksum
    Return bool determines whether or not data is (re)downloaded
    :return: True if local file matches, otherwise False
    """
    try:
        with open(dl_path, "rb") as f:
            # Local file checksum
            actual = hashlib.md5(f.read()).hexdigest()
    except EnvironmentError:
        # Return False if file doesn't exit
        return False

    expected = requests.get("%s.checksum" % url).text.strip()
    return actual == expected


def split_elements(compound):
    """
    Split source tsv elements at colon
    e.g.: 'rel:etymology' => ['rel', 'etymology']
    :return: Elements as list
    """
    elements = [e.strip() for e in compound.split(":")]
    if len(elements) == 2:
        return elements

    result = [elements[0], ":".join(elements[1:])]
    return result


def generate_json(source_path, dir):
    """
    Reads source tsv and restructures data as described:
    https://github.com/jmsv/ety-python/issues/24
    """
    result = {}

    print("Loading source tsv")
    with io.open(source_path, "r", newline="", encoding="utf-8") as source:
        reader = csv.reader(source, delimiter="\t")
        source_rows = list(reader)

    gc.collect()

    print("Structuring data")
    for row in progress.bar(source_rows):
        source_lang, source_word = split_elements(row[0])

        if source_lang not in result:
            result[source_lang] = {}
        if source_word not in result[source_lang]:
            result[source_lang][source_word] = []

        dest_lang, dest_word = split_elements(row[2])
        result[source_lang][source_word].append({dest_word: dest_lang})

        del source_lang, source_word, dest_lang, dest_word

    # Save data to seperate files for languages, may be required in the future
    # print('Saving language files')
    # for key in progress.bar(result):
    #     with io.open(os.path.join(dir, 'data/ety-%s.json' % key), 'w') as f:
    #         f.write(json.dumps(result[key], sort_keys=False))

    # Save data
    print("Writing etymologies file")
    with io.open(os.path.join(dir, "etymologies.json"), "w") as f:
        json.dump(result, f)


def main():
    """
    Define paths, download data if required, generate json dataset
    """
    dir = os.path.dirname(os.path.realpath(__file__))
    source_dir = os.path.join(dir, "source")
    source_path = os.path.join(source_dir, "etymwn.tsv")
    source_url = "https://data.jmsv.me/etymwn-filtered.tsv"

    # Exit if not Python 3
    if not six.PY3:
        print("Script should be run as Python 3, exiting")
        exit(1)

    prepare(source_dir)

    # (Re)download data if required
    if not verify_local_data(source_url, source_path):
        print("Downloading source data")
        download_dataset(source_url, source_path)

        # If checksum still doesn't match, exit
        if verify_local_data(source_url, source_path):
            print("Verified local source data")
        else:
            print("Error verifying local source data, exiting")
            exit(1)
    else:
        print("Verified local source data")

    generate_json(source_path, dir)

    print("Done")


if __name__ == "__main__":
    main()



#!/usr/local/bin/python
# -*- coding: utf-8 -*-
from __future__ import print_function

import argparse

import ety


def cli():
    """
    Command line interface
    :return: Exit code
    """
    parser = argparse.ArgumentParser(prog="ety")
    parser.add_argument("words", type=str, nargs="+", help="the search word(s)")
    parser.add_argument(
        "-r", "--recursive", help="search origins recursively", action="store_true"
    )
    parser.add_argument(
        "-t", "--tree", help="display etymology tree", action="store_true"
    )

    args = parser.parse_args()

    output = ""
    for word in args.words:
        source_word = ety.Word(word, color=True)
        roots = ety.origins(word, recursive=args.recursive)

        if not roots:
            print("No origins found for word: {}".format(word))
            continue

        # Bullet point: '\u2022'
        if args.tree:
            output += "%s\n\n" % str(ety.tree(source_word))
        else:
            output += "\n\n%s\n \u2022 " % source_word
            output += "\n \u2022 ".join(root.pretty for root in roots)

    print(output.strip())

    return 0



#!/usr/local/bin/python
# -*- coding: utf-8 -*-

import re

from six import string_types

from .word import Word


class Census(object):
    def __init__(self, words, language="eng"):
        self.words = []
        self._origins = {"direct": [], "recursive": []}

        if isinstance(words, string_types):
            words = list(re.split(r"\s+", words))  # Split words by whitespace
        elif isinstance(words, (list, tuple)):
            words = list(words)
        else:
            raise ValueError("words argument must be either string or list")

        for word in list(words):
            if isinstance(word, string_types):
                self.words.append(Word(word, language))
            elif isinstance(word, Word):
                self.words.append(word)
            else:
                raise ValueError(
                    "Invalid word type: '%s'. Words must "
                    "be ety.Word objects or strings" % str(type(word))
                )

    def origins(self, recursive=False):
        search = "recursive" if recursive else "direct"

        o = self._origins[search]

        # Return cached direct or recursive origin searches if already searched
        if o:
            return o

        for word in self.words:
            o.extend(word.origins(recursive=recursive))

        return o

    def __eq__(self, other):
        return self.words == other.words



#!/usr/local/bin/python
# -*- coding: utf-8 -*-

import io
import json

from pkg_resources import resource_filename


def load_json_resource(resource):
    resource = resource_filename("ety", resource)
    with io.open(resource, "r", encoding="utf-8") as f:
        return json.load(f)


def load_country_codes(iso_version):
    return load_json_resource("data/iso-{}.json".format(iso_version))


etyms = load_json_resource("data/etymologies.json")
iso_639_3_codes = load_country_codes("639-3")
iso_639_2_codes = load_country_codes("639-2")



#!/usr/local/bin/python
# -*- coding: utf-8 -*-

import colorful
from six import string_types

from .data import etyms as etymwn_data
from .language import Language
from .tree import EtyTree


class Word(object):
    def __init__(self, word, language="eng", color=False):
        if not isinstance(word, string_types):
            raise TypeError("word must be a string")
        self._word = word

        if isinstance(language, Language):
            self._language = language
        else:
            self._language = Language(language)

        self.color = bool(color)

        self._origins = {"direct": [], "recursive": []}
        self._id = u"{}:{}".format(self.word, self.language.iso)

    def origins(self, recursive=False):
        search = "recursive" if recursive else "direct"

        if (
            self.language.iso not in etymwn_data
            or self.word not in etymwn_data[self.language.iso]
        ):
            # There are no roots for this word
            return []

        roots = [
            Word(word, lang)
            for root in etymwn_data[self.language.iso][self.word]
            for word, lang in root.items()
        ]

        tracked = roots[:]

        if recursive:
            for root in tracked:
                for child in root.origins():
                    # Check word isn't already in tree before appending
                    if child not in tracked and child != self:
                        tracked.append(child)

        self._origins[search] = tracked
        return self._origins[search]

    def tree(self):
        return EtyTree(self)

    @property
    def word(self):
        return self._word

    @property
    def language(self):
        return self._language

    @property
    def pretty(self):
        word = colorful.bold(self.word) if self.color else self.word
        return u"{word} ({lang})".format(word=word, lang=self.language.name)

    def __lt__(self, other):
        if isinstance(other, Word):
            return self.pretty < other.pretty
        return self.pretty < other

    def __eq__(self, other):
        if isinstance(other, Word):
            return self._id == other._id
        return self.pretty == other

    def __str__(self):
        return self.pretty

    def __repr__(self):
        return u"Word({word}, {lang} [{iso}])".format(
            word=self.word, lang=self.language, iso=self.language.iso
        )

    def __len__(self):
        return len(self.word)

    @property
    def __dict__(self):
        return {"id": self._id, "word": self.word, "language": self.language.__dict__}



#!/usr/local/bin/python
# -*- coding: utf-8 -*-

import re
from codecs import open
from os import path

from setuptools import setup

here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

with open(path.join(here, "ety/__init__.py"), encoding="utf8") as f:
    version = re.search(r'__version__ = "(.*?)"', f.read()).group(1)

setup(
    name="ety",
    version=version,
    description="discover the etymology of words",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jmsv/ety-python",
    author="James Vickery",
    author_email="dev@jamesvickery.net",
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, <4",
    keywords="etymology origins english language words",
    packages=["ety", "ety/data"],
    install_requires=["treelib", "colorful", "six"],
    extras_require={"dev": ["flake8"]},
    package_data={
        "ety": ["data/etymologies.json", "data/iso-639-3.json", "data/iso-639-2.json"]
    },
    entry_points={"console_scripts": ["ety=ety:cli"]},
    project_urls={
        "Source": "https://github.com/jmsv/ety-python",
        "Bug Reports": "https://github.com/jmsv/ety-python/issues",
    },
)



#!/usr/local/bin/python
# -*- coding: utf-8 -*-

from random import choice

from . import data
from .census import Census
from .cli import cli  # noqa: F401
from .tree import EtyTree
from .word import Word, Language  # noqa: F401


__version__ = "1.3.2"


def _get_source_word(word, language, color=False):
    if isinstance(word, Word):
        return word
    return Word(word, language, color=color)


def origins(word, language="eng", recursive=False, color=False):
    source_word = _get_source_word(word, language, color)
    return source_word.origins(recursive)


def tree(word, language="eng", color=False):
    source_word = _get_source_word(word, language, color)
    return EtyTree(source_word)


def random_word(language="eng"):
    w = choice(list(data.etyms[language]))
    return Word(w, language)


def census(words):
    return Census(words)
